#include <stdlib.h>
#include "arguments.h"

#define SysFilterExclusiveError "--system and --no-system are mutually exclusive"
#define FormatExclusiveError "--json, --list-paths and --list-names are mutually exclusive"

const char *argp_program_version = "peldd 0.3.0";
const char *argp_program_bug_address = "tblFlip <root@tblflip.de>";

static Arguments arguments = {NULL};

static struct argp_option options[] = {
        {"system",               's', NULL, OPTION_ARG_OPTIONAL, "Only output system DLLs",                                     0},
        {"no-system",            'S', NULL, OPTION_ARG_OPTIONAL, "Do not output any system DLLs",                               0},
        {"recursive",            'R', NULL, OPTION_ARG_OPTIONAL, "Resolve dependencies recursively",                            1},
        {"color",                'c', NULL, OPTION_ARG_OPTIONAL, "Colorize output with ANSI colors",                            2},
        {"indent",               'i', "indentation", 0,          "Indent JSON output by a number of spaces",                    2},
        {"json",                 'j', NULL, OPTION_ARG_OPTIONAL, "Output dependencies in JSON format",                          2},
        {"list-paths",           'p', NULL, OPTION_ARG_OPTIONAL, "Only output a list of DLL paths",                             3},
        {"list-names",           'n', NULL, OPTION_ARG_OPTIONAL, "Only output a list of DLL names",                             3},
        {"err-unknown-location", 'e', NULL, OPTION_ARG_OPTIONAL, "Abort the program with an error if a DLL can not be located", 4},
        {NULL}
};

static error_t parseOpt(int key, char *argument, struct argp_state *state) {
    Arguments *args = state->input;

    switch (key) {
        case 's':
            if (args->noSystem) argp_error(state, SysFilterExclusiveError);
            args->onlySystem = true;
            break;

        case 'S':
            if (args->onlySystem) argp_error(state, SysFilterExclusiveError);
            args->noSystem = true;
            break;

        case 'R':
            args->recursive = true;
            break;

        case 'i':
            args->indent = atoi(argument);
            break;

        case 'c':
            args->colorOutput = true;
            break;

        case 'j':
            if (args->listPaths || args->listNames) argp_error(state, FormatExclusiveError);
            args->json = true;
            break;

        case 'p':
            if (args->json || args->listNames) argp_error(state, FormatExclusiveError);
            args->listPaths = true;
            break;

        case 'n':
            if (args->json || args->listPaths)argp_error(state, FormatExclusiveError);
            args->listNames = true;
            break;

        case 'e':
            args->errUnknownLocation = true;
            break;

        case ARGP_KEY_ARG:
            if (state->arg_num > 1) argp_usage(state);
            arguments.peFilePath = argument;
            break;

        case ARGP_KEY_END:
            if (state->arg_num < 1) argp_usage(state);
            break;

        default:
            return ARGP_ERR_UNKNOWN;
    }

    return 0;
}

static struct argp argp = {
        .options = options,
        .parser = parseOpt,
        .args_doc = "<PE Executable Path>",
        .doc = "DLL dependency resolver for PE32(+) executables"
};

Arguments *parseArguments(int argc, char **argv) {
    if (arguments.peFilePath != NULL) errx(1, "Attempted to parse arguments more than once");
    argp_parse(&argp, argc, argv, 0, 0, &arguments);
    return &arguments;
}

Arguments *useArguments() {
    if (arguments.peFilePath == NULL) errx(1, "Arguments have never been parsed correctly");
    return &arguments;
}
