#include <string.h>
#include "deferable.h"

const char DeferableMagic[8] = "DEFER\0ME";

void deferFunc(void *voidPtrPtr) {
    Deferable *deferable = *(Deferable **) voidPtrPtr;
    if (deferable == NULL) return;

    if (isDeferable(deferable)) freeDeferable(deferable);
    else free(deferable);
}

void *allocateDeferable(int count, int structSize, const DeferableFunctionsMap *functions) {
    Deferable *deferable = calloc(count, structSize);

    for (int index = 0; index < count; index++) {
        memcpy((char *) deferable[index].magic, DeferableMagic, 8);
        deferable[index].functions = functions;
        deferable[index].size = structSize;

        if (functions->initFunction != NULL) functions->initFunction(&deferable[index]);
    }

    return deferable;
}

void *cloneDeferable(Deferable *deferable) {
    if (!isDeferable(deferable)) return NULL;
    Deferable *cloned = calloc(1, deferable->size);

    if (deferable->functions->copyFunction != NULL) deferable->functions->copyFunction(deferable, cloned);
    else memcpy(cloned, deferable, deferable->size);

    return cloned;
}

void freeDeferable(void *deferablePtr) {
    Deferable *deferable = deferablePtr;
    if (!isDeferable(deferable)) return;
    if (deferable->error != NULL) free(deferable->error);
    if (deferable->functions->freeFunction != NULL) deferable->functions->freeFunction(deferable);
    free(deferable);
}

bool isDeferable(Deferable *deferable) {
    return memcmp(deferable->magic, DeferableMagic, 8) == 0;
}
