#pragma once

#include <malloc.h>
#include <stdbool.h>
#include <stddef.h>
#include <stdio.h>
#include "util.h"

#define DefineDeferable(deferableStruct) typedef struct deferableStruct deferableStruct;                      \
__attribute__((__weak__)) void init##deferableStruct(deferableStruct *newStruct);                             \
__attribute__((__weak__)) void copy##deferableStruct(deferableStruct *oldStruct, deferableStruct *newStruct); \
__attribute__((__weak__)) void free##deferableStruct(deferableStruct *);                                      \
static const DeferableFunctionsMap deferableStruct##Functions = {                                             \
    .initFunction = (DeferableInitFunction) init##deferableStruct,                                            \
    .copyFunction = (DeferableCopyFunction) copy##deferableStruct,                                            \
    .freeFunction = (DeferableFreeFunction) free##deferableStruct                                             \
};                                                                                                            \
struct deferableStruct

#define dalloc(deferableStruct) ((deferableStruct *) allocateDeferable(1, sizeof(deferableStruct), &deferableStruct##Functions))
#define defer __attribute__((__cleanup__(deferFunc)))

#define checkError(deferable, condition, format, ...) do {                       \
    if (!(condition)) {                                                          \
        ((Deferable *) (deferable))->error = allocFormat((format), __VA_ARGS__); \
        return deferable;                                                        \
    }                                                                            \
} while(0)

#define noError(deferable) if (((Deferable *) (deferable))->error == NULL)
#define check(deferable) if (((Deferable *) (deferable))->error != NULL)
#define errStr(deferable) (((Deferable *) (deferable))->error)

#define fatal(deferable, format, ...) do {           \
    check((deferable)) errx(1, format, __VA_ARGS__); \
} while(0)

#define checkElse(deferable, format, ...) check((deferable)) fprintf(stderr, format "\n", __VA_ARGS__); else

typedef void (*DeferableInitFunction)(void *newPtr);

typedef void (*DeferableCopyFunction)(void *oldPtr, void *newPtr);

typedef void (*DeferableFreeFunction)(void *objPtr);

typedef struct {
    const DeferableInitFunction initFunction;
    const DeferableFreeFunction freeFunction;
    const DeferableCopyFunction copyFunction;
} DeferableFunctionsMap;

typedef struct {
    const char magic[8];
    const DeferableFunctionsMap *functions;
    int size;
    char *error;
} Deferable;

void deferFunc(void *voidPtrPtr);

void *allocateDeferable(int count, int structSize, const DeferableFunctionsMap *functions);

void *cloneDeferable(Deferable *deferable);

void freeDeferable(void *deferable);

bool isDeferable(Deferable *deferable);
