#include "deferable_string.h"

static int stringLengthOrCheckFormat(const char *formatSuspect) {
    int length = 0;

    for (; formatSuspect[length] != '\0'; length++) {
        if (formatSuspect[length] == '%') {
            return -1;
        }
    }

    return length;
}

String *string(const char *format, ...) {
    va_list args;

    String *string = dalloc(String);
    string->length = stringLengthOrCheckFormat(format);

    if (string->length == -1) {
        va_start(args, format);
        string->length = vsnprintf(NULL, 0, format, args);
        va_end(args);

        string->ownsString = true;
        string->cStr = calloc(string->length + 1, sizeof(char));

        va_start(args, format);
        vsnprintf((char *) string->cStr, string->length + 1, format, args);
        va_end(args);
    } else {
        string->cStr = format;
    }

    return string;
}

String *stringDup(const String *string) {
    String *dup = dalloc(String);
    dup->ownsString = true;
    dup->length = string->length;
    dup->cStr = strdup(string->cStr);
    return dup;
}

int compareString(String *a, String *b) {
    return strncmp(a->cStr, b->cStr, min(a->length, b->length));
}

void copyString(String *oldString, String *newString) {
    if (oldString->ownsString) {
        newString->cStr = strndup(oldString->cStr, oldString->length);
    } else {
        newString->cStr = oldString->cStr;
    }

    newString->length = oldString->length;
    newString->ownsString = oldString->ownsString;
}

void freeString(String *string) {
    if (string->ownsString && string->cStr != NULL) {
        free((void *) string->cStr);
    }
}
