#include "import_lookup_table.h"

ImportLookupTable *newImportLookupTable(PeHeader *peHeader, long rva) {
    ImportLookupTable *ilt = dalloc(ImportLookupTable);
    ilt->peHeader = peHeader;
    ilt->section = findSectionByRva(peHeader, rva);
    ilt->offset = 0;

    if (peHeader->peOptionalHeader != NULL) ilt->iltBase32 = (uint32_t *) resolveSectionRva(ilt->section, rva);
    else ilt->iltBase64 = (uint64_t *) resolveSectionRva(ilt->section, rva);

    return ilt;
}

bool hasNextHint(ImportLookupTable *ilt) {
    if (ilt->iltBase32 != NULL) {
        while (1) {
            uint32_t current = *(ilt->iltBase32 + ilt->offset);
            if (current == 0) return false;
            if ((current & OrdinalFlag32) == 0) return true;
            ilt->offset++;
        }
    } else {
        while (1) {
            uint64_t current = *(ilt->iltBase64 + ilt->offset);
            if (current == 0) return false;
            if ((current & OrdinalFlag64) == 0) return true;
            ilt->offset++;
        }
    }
}

HintNameTable *getNextHint(ImportLookupTable *ilt) {
    HintNameTable *hintNameTable;

    if (ilt->iltBase32 != NULL) {
        long hintRva = *(ilt->iltBase32 + ilt->offset++);
        hintNameTable = parseHintNameTable(resolveSectionRva(ilt->section, hintRva));
    } else {
        long hintRva = (long) *(ilt->iltBase64 + ilt->offset++);
        hintNameTable = parseHintNameTable(resolveSectionRva(ilt->section, hintRva));
    }

    return hintNameTable;
}
