#include "mapped_file.h"

MappedFile *newMappedFile(const char *filePath) {
    MappedFile *mappedFile = dalloc(MappedFile);
    mappedFile->filePath = filePath;

    mappedFile->fileDescriptor = open(mappedFile->filePath, O_RDONLY);
    checkError(mappedFile, mappedFile->fileDescriptor != -1, "Unable to open file: %s", strerror(errno));

    mappedFile->fileSize = lseek(mappedFile->fileDescriptor, 0, SEEK_END);
    mappedFile->map = mmap(NULL, mappedFile->fileSize, PROT_READ, MAP_PRIVATE, mappedFile->fileDescriptor, 0);
    checkError(mappedFile, mappedFile != MAP_FAILED, "Unable to mmap file: %s", strerror(errno));

    return mappedFile;
}

void freeMappedFile(MappedFile *mappedFile) {
    if (mappedFile->map != MAP_FAILED) {
        munmap(&mappedFile->map, mappedFile->fileSize);
    }

    if (mappedFile->fileDescriptor != -1) {
        close(mappedFile->fileDescriptor);
    }
}
