#include "pe_header.h"

#define atOffset(type, offset) ((type *) (((uint64_t) peHeader->peFile->map) + (offset)))

PeHeader *parsePeHeader(MappedFile *mappedFile) {
    PeHeader *peHeader = dalloc(PeHeader);
    peHeader->peFile = mappedFile;
    peHeader->dosHeader = mappedFile->map;

    checkError(peHeader, peHeader->dosHeader->signature == DOS_SIGNATURE,
               "Invalid DOS header found: %02X", peHeader->dosHeader->signature);

    peHeader->coffHeader = atOffset(CoffHeader, peHeader->dosHeader->peHeaderOffset);

    checkError(peHeader, peHeader->coffHeader->signature == COF_SIGNATURE,
               "Invalid COFF header found at %08X: %08X", peHeader->dosHeader->peHeaderOffset,
               peHeader->coffHeader->signature);

    unsigned long optHeaderOffset = peHeader->dosHeader->peHeaderOffset + sizeof(CoffHeader);
    unsigned long sectionTableOffset = optHeaderOffset;

    if (peHeader->coffHeader->optionalHeaderSize > 0) {
        uint16_t *magic = atOffset(uint16_t, optHeaderOffset);

        checkError(peHeader, *magic == PE_MAGIC || *magic == PEP_MAGIC,
                   "Invalid PE magic found at %08X: %02X", optHeaderOffset, *magic);

        if (*magic == PE_MAGIC) {
            peHeader->peOptionalHeader = atOffset(PeOptionalHeader, optHeaderOffset);
            sectionTableOffset += sizeof(PeOptionalHeader);
        } else if (*magic == PEP_MAGIC) {
            peHeader->pePlusOptionalHeader = atOffset(PePlusOptionalHeader, optHeaderOffset);
            sectionTableOffset += sizeof(PePlusOptionalHeader);
        }
    }

    peHeader->sectionTable = atOffset(SectionHeader, sectionTableOffset);
    return peHeader;
}

const DataDirectories *getDataDirectories(PeHeader *peHeader) {
    if (peHeader->peOptionalHeader != NULL) return &peHeader->peOptionalHeader->directories;
    else if (peHeader->pePlusOptionalHeader != NULL) return &peHeader->pePlusOptionalHeader->directories;
    else return NULL;
}
