#pragma once

#include <stdint.h>
#include "deferable.h"
#include "mapped_file.h"

#define DOS_SIGNATURE 0x5A4D
#define COF_SIGNATURE 0x4550
#define PE_MAGIC 0x10b
#define PEP_MAGIC 0x20b

typedef struct {
    int16_t signature;
    int8_t padding0[58];
    int32_t peHeaderOffset;
} __attribute__((__packed__)) DosHeader;

typedef struct {
    int32_t signature;
    int16_t machine;
    int16_t nSections;
    int32_t dateTimeStamp;
    int32_t symbolTableOffset;
    int32_t nSymbolTables;
    int16_t optionalHeaderSize;
    int16_t characteristics;
} __attribute__((__packed__)) CoffHeader;

typedef struct {
    int16_t magic;
    int8_t majorLinkerVersion;
    int8_t minorLinkerVersion;
    int32_t sizeOfCode;
    int32_t sizeOfInitializedData;
    int32_t sizeOfUninitializedData;
    int32_t entryPointOffset;
    int32_t codeBaseOffset;
    int32_t dateBaseOffset;
} __attribute__((__packed__)) PeCoffFields;

typedef struct {
    int16_t magic;
    int8_t majorLinkerVersion;
    int8_t minorLinkerVersion;
    int32_t sizeOfCode;
    int32_t sizeOfInitializedData;
    int32_t sizeOfUninitializedData;
    int32_t entryPointOffset;
    int32_t codeBaseOffset;
} __attribute__((__packed__)) PePlusCoffFields;

typedef struct {
    int32_t imageBase;
    int32_t sectionAlignment;
    int32_t fileAlignment;
    int16_t majorOsVersion;
    int16_t minorOsVersion;
    int16_t majorImageVersion;
    int16_t minorImageVersion;
    int16_t majorSubsystemVersion;
    int16_t minorSubsystemVersion;
    int padding0;
    int32_t sizeOfImage;
    int32_t sizeOfHeaders;
    int32_t checkSum;
    int16_t subsystem;
    int16_t dllCharacteristics;
    int32_t sizeOfStackReserved;
    int32_t sizeOfStackCommit;
    int32_t sizeOfHeapReserved;
    int32_t sizeOfHeapCommit;
    int padding1;
    int32_t nRvaAndSizes;
} __attribute__((__packed__)) PeWindowsFields;

typedef struct {
    int64_t imageBase;
    int32_t sectionAlignment;
    int32_t fileAlignment;
    int16_t majorOsVersion;
    int16_t minorOsVersion;
    int16_t majorImageVersion;
    int16_t minorImageVersion;
    int16_t majorSubsystemVersion;
    int16_t minorSubsystemVersion;
    int padding0;
    int32_t sizeOfImage;
    int32_t sizeOfHeaders;
    int32_t checkSum;
    int16_t subsystem;
    int16_t dllCharacteristics;
    int64_t sizeOfStackReserved;
    int64_t sizeOfStackCommit;
    int64_t sizeOfHeapReserved;
    int64_t sizeOfHeapCommit;
    int padding1;
    int32_t nRvaAndSizes;
} __attribute__((__packed__)) PePlusWindowsFields;

typedef struct {
    int32_t offset;
    int32_t size;
} __attribute__((__packed__)) DataDirectory;

typedef struct {
    DataDirectory exportTable;
    DataDirectory importTable;
    DataDirectory resourceTable;
    DataDirectory exceptionTable;
    DataDirectory certificateTable;
    DataDirectory baseRelocationTable;
    DataDirectory debug;
    DataDirectory architecture;
    DataDirectory globalPtr;
    DataDirectory tlsTable;
    DataDirectory loadConfigTable;
    DataDirectory boundImport;
    DataDirectory iat;
    DataDirectory delayImportDescriptor;
    DataDirectory clrRuntimeHeader;
    DataDirectory reserved;
} __attribute__((__packed__)) DataDirectories;

typedef struct {
    PeCoffFields coff;
    PeWindowsFields windows;
    DataDirectories directories;
} __attribute__((__packed__)) PeOptionalHeader;

typedef struct {
    PePlusCoffFields coff;
    PePlusWindowsFields windows;
    DataDirectories directories;
} __attribute__((__packed__)) PePlusOptionalHeader;

typedef struct {
    const char name[8];
    int32_t virtualSize;
    int32_t virtualAddress;
    int32_t rawDataSize;
    int32_t rawDataPtr;
    int32_t relocationsPtr;
    int32_t lineNumbersPtr;
    int16_t nRelocations;
    int16_t nLineNumbers;
    int32_t characteristics;
} __attribute__((__packed__)) SectionHeader;

DefineDeferable(PeHeader){
    Deferable deferable;
    const MappedFile *peFile;
    const DosHeader *dosHeader;
    const CoffHeader *coffHeader;
    const PeOptionalHeader *peOptionalHeader;
    const PePlusOptionalHeader *pePlusOptionalHeader;
    const SectionHeader *sectionTable;
};

PeHeader *parsePeHeader(MappedFile *mappedFile);

const DataDirectories *getDataDirectories(PeHeader*peHeader);
