#include "section.h"

#define atOffset(type, offset) ((type *) (((uint64_t) section->peHeader->peFile->map) + (offset)))
#define atDataOffset(type, offset) ((type *) (((uint64_t) section->data) + (offset)))

Section *findSection(const PeHeader *peHeader, const char *name) {
    Section *section = dalloc(Section);
    section->peHeader = peHeader;

    for (int sectionIndex = 0; sectionIndex < section->peHeader->coffHeader->nSections; sectionIndex++) {
        const SectionHeader *header = &section->peHeader->sectionTable[sectionIndex];

        if (strncmp(header->name, name, 8) == 0) {
            section->header = header;
        }
    }

    checkError(section, section->header != NULL, "No section with name %s exists", name);
    section->data = atOffset(void, section->header->rawDataPtr);
    return section;
}

Section *findSectionByRva(const PeHeader *peHeader, long rva) {
    Section *section = dalloc(Section);
    section->peHeader = peHeader;

    for (int index = 0; index < peHeader->coffHeader->nSections; index++) {
        const SectionHeader *header = &peHeader->sectionTable[index];

        if (rva >= header->virtualAddress && rva <= header->virtualAddress + header->virtualSize) {
            section->header = header;
        }
    }

    checkError(section, section->header != NULL, "No section containing RVA 0x%016lX exists", rva);
    section->data = atOffset(void, section->header->rawDataPtr);
    return section;
}

const void *resolveSectionRva(const Section *section, long rva) {
    return atDataOffset(const void, rva - section->header->virtualAddress);
}
